// Generated by CoffeeScript 2.1.1
(function() {
  var EventEmitter, WebSocket, getPresenceSubscriptionJson, parseMessage, request, rtm, sendPresenceEvent, slackData, userIdToNick;

  request = require('request');

  WebSocket = require('ws');

  EventEmitter = require('events').EventEmitter;

  rtm = new EventEmitter;

  slackData = {};

  //eventemitter FTW!
  //parses an RTM event
  parseMessage = function(msg) {
    msg = JSON.parse(msg);
    //we are only interested in presence change
    if (msg.type === 'presence_change') {
      sendPresenceEvent(msg);
    } else if (msg.type === 'hello') {
      rtm.emit('ready');
    } else {
      rtm.emit(msg.type, msg);
    }
    rtm.emit('*', msg);
  };

  //translates a slack unique userid to readable username
  userIdToNick = function(userid) {
    var i, len, user, users;
    users = slackData.users;
    for (i = 0, len = users.length; i < len; i++) {
      user = users[i];
      if (user.id === userid) {
        return user.name;
      }
    }
    return 'unknown';
  };

  //throw the initial data to the database
  //via the events
  sendPresenceEvent = function(msg) {
    var i, len, ref, user;
    ref = msg.users;
    for (i = 0, len = ref.length; i < len; i++) {
      user = ref[i];
      //ignore the slackbot
      if (user.id !== 'USLACKBOT') {
        rtm.emit(msg.presence, userIdToNick(user));
      }
    }
  };

  getPresenceSubscriptionJson = function(users) {
    return JSON.stringify({
      type: "presence_sub",
      ids: users.map(function(user) {
        return user.id;
      })
    });
  };

  module.exports = function(API_TOKEN) {
    //First call the rtm.start method of Slack API
    request.get({
      url: 'https://slack.com/api/rtm.start?batch_presence_aware=1&token=' + API_TOKEN,
      json: true
    }, function(err, res, json) {
      var ws;
      if (err) {
        throw err;
      }
      //This is the initial response data with a lot of interesting keys
      //So we store it as well
      slackData = json;
      //Connect the websocket
      ws = new WebSocket(json.url);
      //Sets up the callback for websocket messaging
      ws.on('message', parseMessage);
      ws.on('open', function(socket) {
        return rtm.emit('connect');
      });
      ws.on('error', function(err) {
        return rtm.emit('error', err);
      });
      rtm.on('ready', function() {
        return ws.send(getPresenceSubscriptionJson(slackData.users));
      });
    });
    //Return the eventemitter
    return rtm;
  };

}).call(this);
